--[[
secrad = pi * sec / 30
minrad = pi * min / 30
hourrad = pi * (( hour *60) + min )) / 360


x = sin( secrad ) * width
y = -1 * cos( secrad ) * width

--]]

local AnalogClock = LibStub("AceAddon-3.0"):NewAddon("AnalogClock", "AceTimer-3.0", "AceConsole-3.0")

local frame, tminute, thour, tsecond
local t, sec, mn, secrad, minrad, hourrad
local lastsec, lastmn = -1,-1
local radius

local ssize = 20 -- width of the second hand
local msize = 30 -- width of the minute hand
local hsize = 60 -- width of the hour hand
local mradius -- fraction of total radius that the minute hand is
local hradius -- fraction of total radius that the hour hand is
local db
local nums = {}

local defaults = {
	profile = {
		numbercolor = { r = 1, g = 1, b = 1, a = 1 },
		secondcolor = { r = 1, g = 1, b = 1, a = 1 },
		minutecolor = { r = 1, g = 1, b = 1, a = 1 },
		hourcolor = { r = 1, g = 1, b = 1, a = 1 },
		radius = 30,
		fontsize = 10,
	}
}

local options = {
	type = "group",
	name = "Analog Clock",
	get = function( k ) return db.profile[k.arg].r, db.profile[k.arg].g, db.profile[k.arg].b, db.profile[k.arg].a end,
	set = function( k, r, g, b, a )
		db.profile[k.arg].r = r
		db.profile[k.arg].g = g
		db.profile[k.arg].b = b
		db.profile[k.arg].a = a
		for i = 1, 12 do
			nums[i]:SetTextColor( db.profile.numbercolor.r, db.profile.numbercolor.g, db.profile.numbercolor.b, db.profile.numbercolor.a )
		end
		tsecond:SetVertexColor( db.profile.secondcolor.r, db.profile.secondcolor.g, db.profile.secondcolor.b, db.profile.secondcolor.a )
		tminute:SetVertexColor( db.profile.minutecolor.r, db.profile.minutecolor.g, db.profile.minutecolor.b, db.profile.minutecolor.a )
		thour:SetVertexColor( db.profile.hourcolor.r, db.profile.hourcolor.g, db.profile.hourcolor.b, db.profile.hourcolor.a )
	end,
	args = {
		radius = {
			type = "range",
			name = "Radius",
			desc = "Clock radius",
			arg = "radius",
			min = 10, max = 500, step = 5,
			get = function( k ) return db.profile.radius end,
			set = function( k, v )
				db.profile.radius = v
				radius = v
				mradius = radius * .85
				hradius = radius * .7
				local nrad
				for i = 1, 12 do
					nrad = math.pi * ( i * 60 ) / 360
					nums[i]:ClearAllPoints()
					nums[i]:SetPoint( "CENTER", frame, "CENTER", math.sin(nrad)*radius, math.cos(nrad)*radius)
				end
				lastsec = -1 -- reset so the clock truly updates
				lastmn = -1 
				AnalogClock:UpdateClock()
			end,
			order = 1,
		},
		fontsize = {
			type = "range",
			name = "Fontsize",
			desc = "Fontsize for the numbers",
			arg = "fontsize",
			min = 1, max = 36, step = 1,
			get = function( k ) return db.profile.fontsize end,
			set = function( k, v )
				db.profile.fontsize = v
				local font = GameFontNormalSmall:GetFont()
				for i = 1, 12 do
					nums[i]:SetFont( font, v )
				end
			end,
			order = 2,
		},
		spacer = {
			type = "header",
			name = "",
			order = 3,
		},
		numbercolor = {
			type = "color",
			name = "Numbercolor",
			desc = "Color for the numbers",
			arg = "numbercolor",
			order = 4,
		},
		secondcolor = {
			type = "color",
			name = "Secondcolor",
			desc = "Color for the seconds hand",
			arg = "secondcolor",
			order = 5,
		},
		minutecolor = {
			type = "color",
			name = "Minutecolor",
			desc = "Color for the minutes hand",
			arg = "minutecolor",
			order = 6,
		},
		hourcolor = {
			type = "color",
			name = "Hourcolor",
			desc = "Color for the hours hand",
			arg = "hourcolor",
			order = 7,
		},		
	},
}




function AnalogClock:OnInitialize()
	db = LibStub("AceDB-3.0"):New("AnalogClockDB", defaults, "Default")
	radius = db.profile.radius
	mradius = radius * .85
	hradius = radius * .7
	
	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable("Analog Clock", options)
	if GetSpellInfo then
		LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Analog Clock", "Analog Clock")
	else
		self:RegisterChatCommand("analog", function() LibStub("AceConfigDialog-3.0"):Open("Analog Clock") end )	
	end		
end

function AnalogClock:OnEnable()
	self:CreateClock()
	self:ScheduleRepeatingTimer("UpdateClock", .1)
end

function AnalogClock:CreateClock()
	if frame then return end

	frame = CreateFrame("Button", nil, UIParent)

	frame:SetWidth( radius * 2 )
	frame:SetHeight( radius * 2 )
	frame:ClearAllPoints()
	frame:SetPoint("CENTER", UIParent, "CENTER", 0, 0)

	frame:EnableMouse(true)
	frame:RegisterForDrag("LeftButton")
	frame:RegisterForClicks("AnyUp")
	frame:SetMovable(true)
	frame:SetScript("OnDragStart", function() if IsAltKeyDown() then this:StartMoving() end end)
	frame:SetScript("OnDragStop", function() this:StopMovingOrSizing() self:SavePosition() end)

	local col = db.profile.secondcolor
	tsecond = frame:CreateTexture( nil, "ARTWORK")
	tsecond:SetTexture( "Interface\\AddOns\\AnalogClock\\line" )
	tsecond:SetVertexColor( col.r, col.g, col.b, col.a )

	col = db.profile.minutecolor
	tminute = frame:CreateTexture( nil, "ARTWORK")
	tminute:SetTexture( "Interface\\AddOns\\AnalogClock\\line")
	tminute:SetVertexColor( col.r, col.g, col.b, col.a )

	col = db.profile.hourcolor
	thour = frame:CreateTexture( nil, "ARTWORK")
	thour:SetTexture("Interface\\AddOns\\AnalogClock\\line")
	thour:SetVertexColor( col.r, col.g, col.b, col.a )
	
	local font = GameFontNormalSmall:GetFont()
	local nrad
	col = db.profile.numbercolor
	for i = 1, 12 do
		nums[i] = frame:CreateFontString(nil, "OVERLAY")
		nums[i]:SetFont( font, db.profile.fontsize )
		nums[i]:SetTextColor(col.r, col.g, col.b, col.a)
		nums[i]:ClearAllPoints()
		nrad = math.pi * ( i * 60 ) / 360
		nums[i]:SetPoint( "CENTER", frame, "CENTER", math.sin(nrad)*radius, math.cos(nrad)*radius)
		nums[i]:SetText( i )
	end
	
	self:RestorePosition()
end


function AnalogClock:UpdateClock()
	sec = mod( time(), 60 )
	if sec ~= lastsec then
		secrad = math.pi * sec / 30
		lastsec = sec
		self:DrawHand( tsecond, ssize, math.sin(secrad) * radius, math.cos(secrad)*radius )
		mn = tonumber(date("%M"))
		if mn ~= lastmn then
			lastmn = mn
			minrad = math.pi * mn / 30
			hourrad = math.pi * ((tonumber(date("%I"))*60) + mn)/360
			self:DrawHand( tminute, msize, math.sin(minrad) * mradius, math.cos(minrad)*mradius)
			self:DrawHand( thour, hsize, math.sin(hourrad) * hradius, math.cos(hourrad)*hradius)
		end
	end
end


-- The following function is used with permission from Daniel Stephens <iriel@vigilance-committee.org>
local TAXIROUTE_LINEFACTOR = 128/126; -- Multiplying factor for texture coordinates
local TAXIROUTE_LINEFACTOR_2 = TAXIROUTE_LINEFACTOR / 2; -- Half o that

function AnalogClock:DrawHand( T, w, ex, ey )

	local sx = 0
	local sy = 0

	-- Determine dimensions and center point of line
	local dx,dy = ex - sx, ey - sy
	local cx,cy = (sx + ex) / 2, (sy + ey) / 2

	-- Normalize direction if necessary
	if (dx < 0) then
		dx,dy = -dx,-dy
	end

	-- Calculate actual length of line
	local l = sqrt((dx * dx) + (dy * dy))

	-- Sin and Cosine of rotation, and combination (for later)
	local s,c = -dy / l, dx / l
	local sc = s * c

	-- Calculate bounding box size and texture coordinates
	local Bwid, Bhgt, BLx, BLy, TLx, TLy, TRx, TRy, BRx, BRy
	if (dy >= 0) then
		Bwid = ((l * c) - (w * s)) * TAXIROUTE_LINEFACTOR_2
		Bhgt = ((w * c) - (l * s)) * TAXIROUTE_LINEFACTOR_2
		BLx, BLy, BRy = (w / l) * sc, s * s, (l / w) * sc
		BRx, TLx, TLy, TRx = 1 - BLy, BLy, 1 - BRy, 1 - BLx
		TRy = BRx
	else
		Bwid = ((l * c) + (w * s)) * TAXIROUTE_LINEFACTOR_2
		Bhgt = ((w * c) + (l * s)) * TAXIROUTE_LINEFACTOR_2
		BLx, BLy, BRx = s * s, -(l / w) * sc, 1 + (w / l) * sc
		BRy, TLx, TLy, TRy = BLx, 1 - BRx, 1 - BLx, 1 - BLy
		TRx = TLy
	end

	-- Set texture coordinates and anchors

	T:ClearAllPoints()
	T:SetTexCoord(TLx, TLy, BLx, BLy, TRx, TRy, BRx, BRy)
	T:SetPoint("BOTTOMLEFT", frame, "CENTER", cx - Bwid, cy - Bhgt)
	T:SetPoint("TOPRIGHT", frame, "CENTER", cx + Bwid, cy + Bhgt)

end

function AnalogClock:SavePosition()
	local f = frame
	local s = f:GetEffectiveScale()

	local x, y = f:GetCenter()
	x = x * s
	y = y * s
	
	db.profile.posx = x -- f:GetLeft() * s
	db.profile.posy = y -- f:GetTop() * s
end

function AnalogClock:RestorePosition()
	local x = db.profile.posx
	local y = db.profile.posy

	if not x or not y then return end

	local f = frame
	local s = f:GetEffectiveScale()

	f:ClearAllPoints()
	f:SetPoint("CENTER", UIParent, "BOTTOMLEFT", x / s, y / s)
end
